'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createExperimentUseCase = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _useCase = require('./use-case');

var _useCase2 = _interopRequireDefault(_useCase);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DETECTION_DELAY = 5000;

/**
 * Wais until func returns a value and calls callback with it
 * @param  {Function}   truthyFn       Function to execute to check value
 * @param  {Function} callback       Callback callled passing value
 * @param  {Number}   [delay=100]    Delay allowed to wait for value
 * @param  {Number}   [interval=100] Interval time for checks
 */
var waitUntil = function waitUntil(truthyFn, callback) {
  var delay = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 100;
  var interval = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 100;

  var intervalId = setInterval(function () {
    var value = truthyFn();
    if (value || delay <= 0) {
      clearInterval(intervalId);
      callback(value);
    }
    delay -= interval;
  }, interval);
};

var getOptmizely = function getOptmizely() {
  return window && window.optimizely;
};

var optimizelyPromise = void 0;

var OptimizelyXExperimentsService = function () {
  function OptimizelyXExperimentsService() {
    (0, _classCallCheck3.default)(this, OptimizelyXExperimentsService);
  }

  (0, _createClass3.default)(OptimizelyXExperimentsService, null, [{
    key: 'getSDK',

    /**
     * Get OptimizelyX SDK instance
     * @return Promise<Object>
     */
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                optimizelyPromise = optimizelyPromise || new _promise2.default(function (resolve) {
                  waitUntil(getOptmizely, resolve, DETECTION_DELAY);
                });
                return _context.abrupt('return', optimizelyPromise);

              case 2:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function getSDK() {
        return _ref.apply(this, arguments);
      }

      return getSDK;
    }()

    /**
     * Get info of an experiment
     * @param {String} experimentId
     * @return Promise<Object>
     * @example
     * {
     *  id: '100',
     *  isActive: true,
     *  isInExperimentHoldback: true,
     *  variation: {
     *      id: '101',
     *      name: 'Variation #1'
     *  }
     * }
     */

  }, {
    key: 'getInfo',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(experimentId) {
        var sdk;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return this.getSDK();

              case 2:
                sdk = _context2.sent;
                return _context2.abrupt('return', sdk && sdk.get && sdk.get('state').getExperimentStates()[experimentId]);

              case 4:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function getInfo(_x3) {
        return _ref2.apply(this, arguments);
      }

      return getInfo;
    }()

    /**
     * Get if an experiment is activated for current user
     * @param {String} experimentId
     * @return Promise<Boolean>
     */

  }, {
    key: 'isActivated',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(experimentId) {
        var info;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return this.getInfo(experimentId);

              case 2:
                _context3.t0 = _context3.sent;

                if (_context3.t0) {
                  _context3.next = 5;
                  break;
                }

                _context3.t0 = {};

              case 5:
                info = _context3.t0;
                return _context3.abrupt('return', info.isActive && !info.isInExperimentHoldback || false);

              case 7:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function isActivated(_x4) {
        return _ref3.apply(this, arguments);
      }

      return isActivated;
    }()

    /**
     * Get variation to display for current user
     * @param {String} experimentId
     * @return Promise<String|Number>
     */

  }, {
    key: 'getVariation',
    value: function () {
      var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4(experimentId) {
        var info;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return this.getInfo(experimentId);

              case 2:
                _context4.t0 = _context4.sent;

                if (_context4.t0) {
                  _context4.next = 5;
                  break;
                }

                _context4.t0 = {};

              case 5:
                info = _context4.t0;
                return _context4.abrupt('return', info.variation.id || null);

              case 7:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function getVariation(_x5) {
        return _ref4.apply(this, arguments);
      }

      return getVariation;
    }()
  }]);
  return OptimizelyXExperimentsService;
}();

/**
 * Get instance of use case
 * @param {String} experimentId
 * @return {ExperimentUseCase}
 */


function createExperimentUseCase(experimentId) {
  return new _useCase2.default(OptimizelyXExperimentsService, experimentId);
}

exports.default = OptimizelyXExperimentsService;
exports.createExperimentUseCase = createExperimentUseCase;